package Hash::Check;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;
require AutoLoader;


use Digest::SHA1 qw( sha1 sha1_hex );
use constant RANDOMDEV => "/dev/urandom";


@ISA = qw(Exporter AutoLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw();
$VERSION = '0.02';


# Preloaded methods go here.

sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $self  = {};
	$self->{Version} = $VERSION;
	$self->{To}	= "";
	$self->{Bits} = undef;
	$self->{Comment} = "";
	$self->{Date} = undef;
	$self->{Rand} = "";
	$self->{Hash} = "";
	$self->{Printed} = "";
	bless ($self, $class);
	return $self;
}

sub to {
	my $self = shift;
	if (@_) { $self->{To} = shift; $self->print};
	return $self->{To};
}

sub bits {
	my $self = shift;
	if (@_) { $self->{Bits} = shift; $self->print};
	return $self->{Bits};
}

sub date {
	my $self = shift;
	if (@_) { $self->{Date} = shift; $self->print};
	return $self->{Date};
}

sub comment {
	my $self = shift;
	if (@_) { $self->{Comment} = shift; $self->print};
	return $self->{Comment};
}

sub rand {
	my $self = shift;
	if (@_) { $self->{Rand} = shift; $self->print};
	return $self->{Rand};
}

sub hash {
	my $self = shift;
	if (@_) { $self->{Hash} = shift; $self->print};
	return $self->{Hash};
}

sub print {
	my $self = shift;
	my ($check, $i);
	$check .= "HashCheck\n";
	foreach $i ("Version", "To" ,"Bits" ,"Comment" ,"Date" ,"Rand", "Hash") {
		$check .= "$i: $self->{$i}\n";
	}
	chomp $check;
	$self->{Printed} = $check;
	return $check;
}

sub parse {
	my ($self, $check) = @_;
	my (@fields, $i, $name, $value);
	@fields = split /\n/, $check;
	foreach $i (@fields) {
		($name, $value) = split /:\s+/, $i, 2;
		$self->{$name} = $value if (exists $self->{$name}); 
	}
	$self->{Printed} = $check;
}

sub check {
	my $self = shift;
	if (@_) { my $c = shift; $self->parse($c)};
	return undef if (length($self->{Hash}) < $self->{Bits});
	return 1 if ((unpack "b*", sha1($self->{Printed})) =~ m/$self->{Hash}$/);
	return undef;
}

sub write {
	my $self = shift;
	my ($hash, $h);
	open RAND, RANDOMDEV or die "Couldn't open RANDOMDEV";
	$self->hash ( unpack "b$self->{Bits}", <RAND>);
	while ($h !~ m/$self->{Hash}$/) {
		$self->rand ( unpack "H32", <RAND>);
		$h = unpack "b*", sha1($self->print);
	}
	close RAND;
	return $self;
}
  

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

Hash::Check - Offline HashCash

=head1 SYNOPSIS

  use Hash::Check;

  # write a check

  my $check = new Hash::Check;

  $check->to("someservice@some.service.provider");
  $check->comment("Some comment");
  $check->bits(20);
  $check->date(scalar time);

  $check->write; # This may take some time depending on bits
  my $check_as_string = $check->print;

  # verify a check
  my $check = new Hash::Check;
  $check->parse($check_as_string);
  if ($check->check) {
	&provide_the_service;
  } else {
	print "Forged check!\n";
  }


=head1 DESCRIPTION

A Hash::Check is the offline version of HashCash. It can be
used as a prove of expenditure of computing power. A HashCheck
as string looks like:

   HashCheck
   Version: 0.1
   To: provos@citi.umich.edu
   Bits: 12
   Comment: test
   Date: 1015030975
   Rand: 1530c9285266d00f260983b793861dfd
   Hash: 001110111111

The application fills the I<To>, I<Bits>, I<Date> and I<Comment>
fields. The I<check> method then randomly selects a I<Hash> value
that is I<Bits> bits long. It then tries to find a partial 
collision on the lower I<Bits> bits of the B<SHA1> hash function
by variing the value of I<Rand> until the hash over the whole
string has exactly the I<Hash> bits on the lower end. 

=head1 AUTHOR

Matthias Bauer F<E<lt>matthiasb@acm.orgE<gt>>

=head1 SEE ALSO

L<Digest::SHA1>

Adam Back's HashCash Page <URL:http://www.cypherspace.org/hashcash/> 

=cut
package Hash::Check;

